@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2019 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -22,27 +22,23 @@ sabnzbd.assembler - threaded assembly/decoding of files
 import os
 import Queue
 import logging
-import struct
 import re
 from threading import Thread
 from time import sleep
-try:
-    import hashlib
-    new_md5 = hashlib.md5
-except:
-    import md5
-    new_md5 = md5.new
+import hashlib
 
 import sabnzbd
-from sabnzbd.misc import get_filepath, sanitize_filename, get_unique_filename, renamer, \
-    set_permissions, flag_file, long_path, clip_path, has_win_device, get_all_passwords
-from sabnzbd.constants import QCHECK_FILE, Status
+from sabnzbd.misc import get_filepath, sanitize_filename, set_permissions, \
+    long_path, clip_path, has_win_device, get_all_passwords, diskspace, \
+    get_filename, get_ext, is_rarfile
+from sabnzbd.constants import Status, GIGI
 import sabnzbd.cfg as cfg
 from sabnzbd.articlecache import ArticleCache
 from sabnzbd.postproc import PostProcessor
 import sabnzbd.downloader
+import sabnzbd.par2file as par2file
 import sabnzbd.utils.rarfile as rarfile
-from sabnzbd.encoding import unicoder, deunicode, is_utf8
+from sabnzbd.encoding import unicoder
 from sabnzbd.rating import Rating
 
 
@@ -65,7 +61,6 @@ class Assembler(Thread):
         self.queue.put(job)
 
     def run(self):
-        import sabnzbd.nzbqueue
         while 1:
             job = self.queue.get()
             if not job:
@@ -75,124 +70,123 @@ class Assembler(Thread):
             nzo, nzf = job
 
             if nzf:
-                sabnzbd.CheckFreeSpace()
-                # We allow win_devices because otherwise par2cmdline fails to repair
-                filename = sanitize_filename(nzf.filename, allow_win_devices=True)
-                nzf.filename = filename
+                # Check if enough disk space is free, if not pause downloader and send email
+                if diskspace(force=True)['download_dir'][1] < (cfg.download_free.get_float() + nzf.bytes) / GIGI:
+                    # Only warn and email once
+                    if not sabnzbd.downloader.Downloader.do.paused:
+                        logging.warning(T('Too little diskspace forcing PAUSE'))
+                        # Pause downloader, but don't save, since the disk is almost full!
+                        sabnzbd.downloader.Downloader.do.pause()
+                        sabnzbd.emailer.diskfull()
+                        # Abort all direct unpackers, just to be sure
+                        sabnzbd.directunpacker.abort_all()
 
-                dupe = nzo.check_for_dupe(nzf)
-
-                filepath = get_filepath(long_path(cfg.download_dir.get_path()), nzo, filename)
+                # Prepare filename
+                nzo.verify_nzf_filename(nzf)
+                nzf.filename = sanitize_filename(nzf.filename)
+                filepath = get_filepath(long_path(cfg.download_dir.get_path()), nzo, nzf.filename)
+                nzf.filename = get_filename(filepath)
 
                 if filepath:
                     logging.info('Decoding %s %s', filepath, nzf.type)
                     try:
-                        filepath = _assemble(nzf, filepath, dupe)
+                        filepath = self.assemble(nzf, filepath)
                     except IOError, (errno, strerror):
-                        if nzo.is_gone():
-                            # Job was deleted, ignore error
-                            pass
-                        else:
+                        # If job was deleted or in active post-processing, ignore error
+                        if not nzo.deleted and not nzo.is_gone() and not nzo.pp_active:
                             # 28 == disk full => pause downloader
                             if errno == 28:
                                 logging.error(T('Disk full! Forcing Pause'))
                             else:
                                 logging.error(T('Disk error on creating file %s'), clip_path(filepath))
+                            # Log traceback
+                            logging.info('Traceback: ', exc_info=True)
                             # Pause without saving
-                            sabnzbd.downloader.Downloader.do.pause(save=False)
+                            sabnzbd.downloader.Downloader.do.pause()
+                        continue
                     except:
                         logging.error(T('Fatal error in Assembler'), exc_info=True)
                         break
 
+                    # Clean-up admin data
                     nzf.remove_admin()
-                    setname = nzf.setname
-                    if nzf.is_par2 and (nzo.md5packs.get(setname) is None):
-                        pack = GetMD5Hashes(filepath)[0]
-                        if pack:
-                            nzo.md5packs[setname] = pack
-                            logging.debug('Got md5pack for set %s', setname)
 
-                    rar_encrypted, unwanted_file = check_encrypted_and_unwanted_files(nzo, filepath)
-                    if rar_encrypted:
-                        if cfg.pause_on_pwrar() == 1:
-                            logging.warning(T('WARNING: Paused job "%s" because of encrypted RAR file (if supplied, all passwords were tried)'), nzo.final_name)
-                            nzo.pause()
-                        else:
-                            logging.warning(T('WARNING: Aborted job "%s" because of encrypted RAR file (if supplied, all passwords were tried)'), nzo.final_name)
-                            nzo.fail_msg = T('Aborted, encryption detected')
-                            sabnzbd.nzbqueue.NzbQueue.do.end_job(nzo)
+                    # Do rar-related processing
+                    if is_rarfile(filepath):
+                        # Encryption and unwanted extension detection
+                        rar_encrypted, unwanted_file = check_encrypted_and_unwanted_files(nzo, filepath)
+                        if rar_encrypted:
+                            if cfg.pause_on_pwrar() == 1:
+                                logging.warning(remove_warning_label(T('WARNING: Paused job "%s" because of encrypted RAR file (if supplied, all passwords were tried)')), nzo.final_name)
+                                nzo.pause()
+                            else:
+                                logging.warning(remove_warning_label(T('WARNING: Aborted job "%s" because of encrypted RAR file (if supplied, all passwords were tried)')), nzo.final_name)
+                                nzo.fail_msg = T('Aborted, encryption detected')
+                                sabnzbd.nzbqueue.NzbQueue.do.end_job(nzo)
 
-                    if unwanted_file:
-                        logging.warning(T('WARNING: In "%s" unwanted extension in RAR file. Unwanted file is %s '), nzo.final_name, unwanted_file)
-                        logging.debug(T('Unwanted extension is in rar file %s'), filepath)
-                        if cfg.action_on_unwanted_extensions() == 1 and nzo.unwanted_ext == 0:
-                            logging.debug('Unwanted extension ... pausing')
-                            nzo.unwanted_ext = 1
-                            nzo.pause()
-                        if cfg.action_on_unwanted_extensions() == 2:
-                            logging.debug('Unwanted extension ... aborting')
-                            nzo.fail_msg = T('Aborted, unwanted extension detected')
-                            sabnzbd.nzbqueue.NzbQueue.do.end_job(nzo)
+                        if unwanted_file:
+                            logging.warning(remove_warning_label(T('WARNING: In "%s" unwanted extension in RAR file. Unwanted file is %s ')), nzo.final_name, unwanted_file)
+                            logging.debug(T('Unwanted extension is in rar file %s'), filepath)
+                            if cfg.action_on_unwanted_extensions() == 1 and nzo.unwanted_ext == 0:
+                                logging.debug('Unwanted extension ... pausing')
+                                nzo.unwanted_ext = 1
+                                nzo.pause()
+                            if cfg.action_on_unwanted_extensions() == 2:
+                                logging.debug('Unwanted extension ... aborting')
+                                nzo.fail_msg = T('Aborted, unwanted extension detected')
+                                sabnzbd.nzbqueue.NzbQueue.do.end_job(nzo)
+
+                        # Add to direct unpack
+                        nzo.add_to_direct_unpacker(nzf)
+
+                    elif par2file.is_parfile(filepath):
+                        # Parse par2 files, cloaked or not
+                        nzo.handle_par2(nzf, filepath)
 
                     filter, reason = nzo_filtered_by_rating(nzo)
                     if filter == 1:
-                        logging.warning(Ta('WARNING: Paused job "%s" because of rating (%s)'), nzo.final_name, reason)
+                        logging.warning(remove_warning_label(T('WARNING: Paused job "%s" because of rating (%s)')), nzo.final_name, reason)
                         nzo.pause()
                     elif filter == 2:
-                        logging.warning(Ta('WARNING: Aborted job "%s" because of rating (%s)'), nzo.final_name, reason)
+                        logging.warning(remove_warning_label(T('WARNING: Aborted job "%s" because of rating (%s)')), nzo.final_name, reason)
                         nzo.fail_msg = T('Aborted, rating filter matched (%s)') % reason
                         sabnzbd.nzbqueue.NzbQueue.do.end_job(nzo)
 
-                    nzf.completed = True
             else:
                 sabnzbd.nzbqueue.NzbQueue.do.remove(nzo.nzo_id, add_to_history=False, cleanup=False)
                 PostProcessor.do.process(nzo)
 
+    def assemble(self, nzf, path):
+        """ Assemble a NZF from its table of articles """
+        md5 = hashlib.md5()
+        fout = open(path, 'ab')
+        decodetable = nzf.decodetable
 
-def _assemble(nzf, path, dupe):
-    if os.path.exists(path):
-        unique_path = get_unique_filename(path)
-        if dupe:
-            path = unique_path
-        else:
-            renamer(path, unique_path)
+        for articlenum in decodetable:
+            # Break if deleted during writing
+            if nzf.nzo.status is Status.DELETED:
+                break
 
-    fout = open(path, 'ab')
+            # Sleep to allow decoder/assembler switching
+            sleep(0.0001)
+            article = decodetable[articlenum]
 
-    if cfg.quick_check():
-        md5 = new_md5()
-    else:
-        md5 = None
+            data = ArticleCache.do.load_article(article)
 
-    decodetable = nzf.decodetable
-
-    for articlenum in decodetable:
-        # Break if deleted during writing
-        if nzf.nzo.status is Status.DELETED:
-            break
-
-        # Sleep to allow decoder/assembler switching
-        sleep(0.001)
-        article = decodetable[articlenum]
-
-        data = ArticleCache.do.load_article(article)
-
-        if not data:
-            logging.info(T('%s missing'), article)
-        else:
-            # yenc data already decoded, flush it out
-            fout.write(data)
-            if md5:
+            if not data:
+                logging.info(T('%s missing'), article)
+            else:
+                # yenc data already decoded, flush it out
+                fout.write(data)
                 md5.update(data)
 
-    fout.flush()
-    fout.close()
-    set_permissions(path)
-    if md5:
+        fout.flush()
+        fout.close()
+        set_permissions(path)
         nzf.md5sum = md5.digest()
         del md5
 
-    return path
+        return path
 
 
 def file_has_articles(nzf):
@@ -210,97 +204,26 @@ def file_has_articles(nzf):
     return has
 
 
-# For a full description of the par2 specification, visit:
-# http://parchive.sourceforge.net/docs/specifications/parity-volume-spec/article-spec.html
-
-def GetMD5Hashes(fname, force=False):
-    """ Get the hash table from a PAR2 file
-        Return as dictionary, indexed on names and True for utf8-encoded names
-    """
-    new_encoding = True
-    table = {}
-    if force or not flag_file(os.path.split(fname)[0], QCHECK_FILE):
-        try:
-            f = open(fname, 'rb')
-        except:
-            return table, new_encoding
-
-        new_encoding = False
-        try:
-            header = f.read(8)
-            while header:
-                name, hash = ParseFilePacket(f, header)
-                new_encoding |= is_utf8(name)
-                if name:
-                    table[name] = hash
-                header = f.read(8)
-
-        except (struct.error, IndexError):
-            logging.info('Cannot use corrupt par2 file for QuickCheck, "%s"', fname)
-            table = {}
-        except:
-            logging.debug('QuickCheck parser crashed in file %s', fname)
-            logging.info('Traceback: ', exc_info=True)
-            table = {}
-
-        f.close()
-    return table, new_encoding
-
-
-def ParseFilePacket(f, header):
-    """ Look up and analyze a FileDesc package """
-
-    nothing = None, None
-
-    if header != 'PAR2\0PKT':
-        return nothing
-
-    # Length must be multiple of 4 and at least 20
-    len = struct.unpack('<Q', f.read(8))[0]
-    if int(len / 4) * 4 != len or len < 20:
-        return nothing
-
-    # Next 16 bytes is md5sum of this packet
-    md5sum = f.read(16)
-
-    # Read and check the data
-    data = f.read(len - 32)
-    md5 = new_md5()
-    md5.update(data)
-    if md5sum != md5.digest():
-        return nothing
-
-    # The FileDesc packet looks like:
-    # 16 : "PAR 2.0\0FileDesc"
-    # 16 : FileId
-    # 16 : Hash for full file **
-    # 16 : Hash for first 16K
-    #  8 : File length
-    # xx : Name (multiple of 4, padded with \0 if needed) **
-
-    # See if it's the right packet and get name + hash
-    for offset in range(0, len, 8):
-        if data[offset:offset + 16] == "PAR 2.0\0FileDesc":
-            hash = data[offset + 32:offset + 48]
-            filename = data[offset + 72:].strip('\0')
-            return filename, hash
-
-    return nothing
-
-
 RE_SUBS = re.compile(r'\W+sub|subs|subpack|subtitle|subtitles(?![a-z])', re.I)
-def is_cloaked(path, names):
+SAFE_EXTS = ('.mkv', '.mp4', '.avi', '.wmv', '.mpg', '.webm')
+def is_cloaked(nzo, path, names):
     """ Return True if this is likely to be a cloaked encrypted post """
-    fname = unicoder(os.path.split(path)[1]).lower()
+    fname = unicoder(get_filename(path)).lower()
     fname = os.path.splitext(fname)[0]
     for name in names:
-        name = os.path.split(name.lower())[1]
+        name = get_filename(name.lower())
         name, ext = os.path.splitext(unicoder(name))
         if ext == u'.rar' and fname.startswith(name) and (len(fname) - len(name)) < 8 and len(names) < 3 and not RE_SUBS.search(fname):
-            logging.debug('File %s is probably encrypted due to RAR with same name inside this RAR', fname)
+            # Only warn once
+            if nzo.encrypted == 0:
+                logging.warning(T('Job "%s" is probably encrypted due to RAR with same name inside this RAR'), nzo.final_name)
+                nzo.encrypted = 1
             return True
-        elif 'password' in name:
-            logging.debug('RAR %s is probably encrypted: "password" in filename %s', fname, name)
+        elif 'password' in name and ext not in SAFE_EXTS:
+            # Only warn once
+            if nzo.encrypted == 0:
+                logging.warning(T('Job "%s" is probably encrypted: "password" in filename "%s"'), nzo.final_name, name)
+                nzo.encrypted = 1
             return True
     return False
 
@@ -318,17 +241,18 @@ def check_encrypted_and_unwanted_files(nzo, filepath):
                 return encrypted, unwanted
 
             # Is it even a rarfile?
-            if rarfile.is_rarfile(filepath):
+            if is_rarfile(filepath):
+                # Open the rar
+                rarfile.UNRAR_TOOL = sabnzbd.newsunpack.RAR_COMMAND
                 zf = rarfile.RarFile(filepath, all_names=True)
 
                 # Check for encryption
-                if nzo.encrypted == 0 and cfg.pause_on_pwrar() and (zf.needs_password() or is_cloaked(filepath, zf.namelist())):
+                if nzo.encrypted == 0 and cfg.pause_on_pwrar() and (zf.needs_password() or is_cloaked(nzo, filepath, zf.namelist())):
                     # Load all passwords
                     passwords = get_all_passwords(nzo)
 
                     # Cloaked job?
-                    if is_cloaked(filepath, zf.namelist()):
-                        nzo.encrypted = 1
+                    if is_cloaked(nzo, filepath, zf.namelist()):
                         encrypted = True
                     elif not sabnzbd.HAVE_CRYPTOGRAPHY and not passwords:
                         # if no cryptography installed, only error when no password was set
@@ -339,7 +263,6 @@ def check_encrypted_and_unwanted_files(nzo, filepath):
                     elif sabnzbd.HAVE_CRYPTOGRAPHY:
                         # Lets test if any of the password work
                         password_hit = False
-                        rarfile.UNRAR_TOOL = sabnzbd.newsunpack.RAR_COMMAND
 
                         for password in passwords:
                             if password:
@@ -367,6 +290,9 @@ def check_encrypted_and_unwanted_files(nzo, filepath):
 
                         # Did any work?
                         if password_hit:
+                            # We always trust the user's input
+                            if not nzo.password:
+                                nzo.password = password_hit
                             # Don't check other files
                             logging.info('Password "%s" matches for job "%s"', password_hit, nzo.final_name)
                             nzo.encrypted = -1
@@ -384,13 +310,14 @@ def check_encrypted_and_unwanted_files(nzo, filepath):
                 if cfg.unwanted_extensions() and cfg.action_on_unwanted_extensions():
                     for somefile in zf.namelist():
                         logging.debug('File contains: %s', somefile)
-                        if os.path.splitext(somefile)[1].replace('.', '').lower() in cfg.unwanted_extensions():
+                        if get_ext(somefile).replace('.', '').lower() in cfg.unwanted_extensions():
                             logging.debug('Unwanted file %s', somefile)
                             unwanted = somefile
                 zf.close()
                 del zf
         except:
-            logging.info('Error during inspection of RAR-file %s', filepath, exc_info=True)
+            logging.info('Error during inspection of RAR-file %s', filepath)
+            logging.debug('Traceback: ', exc_info=True)
 
     return encrypted, unwanted
 
@@ -402,11 +329,11 @@ def nzo_filtered_by_rating(nzo):
             nzo.rating_filtered = 1
             reason = rating_filtered(rating, nzo.filename.lower(), True)
             if reason is not None:
-                return (2, reason)
+                return 2, reason
             reason = rating_filtered(rating, nzo.filename.lower(), False)
             if reason is not None:
-                return (1, reason)
-    return (0, "")
+                return 1, reason
+    return 0, ""
 
 
 def rating_filtered(rating, filename, abort):
@@ -434,3 +361,11 @@ def rating_filtered(rating, filename, abort):
     if any(check_keyword(k) for k in keywords.split(',')):
         return T('keywords')
     return None
+
+
+def remove_warning_label(msg):
+    """ Standardize errors by removing obsolete
+        "WARNING:" part in all languages """
+    if ':' in msg:
+        return msg.split(':')[1].strip()
+    return msg
