@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2019 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -25,9 +25,9 @@ POSTPROC_QUEUE_VERSION = 2
 
 REC_RAR_VERSION = 500
 
-PNFO = namedtuple('PNFO', 'repair unpack delete script nzo_id filename password '
-                          'unpackstrht msgid category url bytes_left bytes avg_stamp '
-                          'avg_date finished_files active_files queued_files status priority missing')
+PNFO = namedtuple('PNFO', 'repair unpack delete script nzo_id filename password unpackstrht '
+                          'msgid category url bytes_left bytes avg_stamp avg_date finished_files '
+                          'active_files queued_files status priority bytes_missing direct_unpack')
 
 QNFO = namedtuple('QNFO', 'bytes bytes_left bytes_left_previous_page list q_size_list q_fullsize')
 
@@ -47,25 +47,21 @@ SCAN_FILE_NAME = 'watched_data2.sab'
 FUTURE_Q_FOLDER = 'future'
 JOB_ADMIN = '__ADMIN__'
 VERIFIED_FILE = '__verified__'
-QCHECK_FILE = '__skip_qcheck__'
 RENAMES_FILE = '__renames__'
 ATTRIB_FILE = 'SABnzbd_attrib'
 REPAIR_REQUEST = 'repair-all.sab'
 
-DB_HISTORY_VERSION = 1
-DB_QUEUE_VERSION = 1
+SABYENC_VERSION_REQUIRED = '3.3.5'
 
+DB_HISTORY_VERSION = 1
 DB_HISTORY_NAME = 'history%s.db' % DB_HISTORY_VERSION
-DB_QUEUE_NAME = 'queue%s.db' % DB_QUEUE_VERSION
 
 DEF_DOWNLOAD_DIR = 'Downloads/incomplete'
 DEF_COMPLETE_DIR = 'Downloads/complete'
 DEF_ADMIN_DIR = 'admin'
-DEF_LOG_DIR = 'logs'
 DEF_NZBBACK_DIR = ''
 DEF_LANGUAGE = 'locale'
 DEF_INTERFACES = 'interfaces'
-DEF_INT_LANGUAGE = 'locale'
 DEF_EMAIL_TMPL = 'email'
 DEF_STDCONFIG = 'Config'
 DEF_STDINTF = 'Glitter'
@@ -78,17 +74,15 @@ DEF_WORKDIR = 'sabnzbd'
 DEF_LOG_FILE = 'sabnzbd.log'
 DEF_LOG_ERRFILE = 'sabnzbd.error.log'
 DEF_LOG_CHERRY = 'cherrypy.log'
-DEF_CACHE_LIMIT = '450M'
+DEF_ARTICLE_CACHE_DEFAULT = '500M'
+DEF_ARTICLE_CACHE_MAX = '1G'
 DEF_TIMEOUT = 60
-MIN_TIMEOUT = 10
-MAX_TIMEOUT = 200
-DEF_LOGLEVEL = 1
 DEF_SCANRATE = 5
-DEF_QRATE = 0
 MAX_DECODE_QUEUE = 10
 LIMIT_DECODE_QUEUE = 100
 MAX_WARNINGS = 20
 MAX_WIN_DFOLDER = 60
+MAX_BAD_ARTICLES = 5
 
 REPAIR_PRIORITY = 3
 TOP_PRIORITY = 2
@@ -100,9 +94,10 @@ PAUSED_PRIORITY = -2
 DUP_PRIORITY = -3
 STOP_PRIORITY = -4
 
-STAGES = { 'Source' : 0, 'Download' : 1, 'Servers' : 2, 'Repair' : 3, 'Filejoin' : 4, 'Unpack' : 5, 'Script' : 6 }
+STAGES = {'Source': 0, 'Download': 1, 'Servers': 2, 'Repair': 3, 'Filejoin': 4, 'Unpack': 5, 'Script': 6}
 
 VALID_ARCHIVES = ('.zip', '.rar', '.7z')
+VALID_NZB_FILES = ('.nzb', '.gz', '.bz2')
 
 IGNORED_FOLDERS = ('@eaDir', '.appleDouble')
 
@@ -125,14 +120,14 @@ date_match = [r'(\d{4})\W(\d{1,2})\W(\d{1,2})',  # 2008-10-16
 
 year_match = r'[\W]([1|2]\d{3})([^\w]|$)'  # Something '(YYYY)' or '.YYYY.' or ' YYYY '
 
-sample_match = r'((^|[\W_])sample\d*[\W_])'  # something-sample.avi
+sample_match = r'((^|[\W_])(sample|proof))'  # something-sample or something-proof
 
 
-class Status():
+class Status:
     COMPLETED = 'Completed'         # PP: Job is finished
     CHECKING = 'Checking'           # Q:  Pre-check is running
     DOWNLOADING = 'Downloading'     # Q:  Normal downloading
-    EXTRACTING = 'Extracting'       # PP: Archives are being extraced
+    EXTRACTING = 'Extracting'       # PP: Archives are being extracted
     FAILED = 'Failed'               # PP: Job has failed, now in History
     FETCHING = 'Fetching'           # Q:  Job is downloading extra par2 files
     GRABBING = 'Grabbing'           # Q:  Getting an NZB from an external site
