@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2019 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -28,9 +28,10 @@ import bz2
 import threading
 
 import sabnzbd
-from sabnzbd.constants import SCAN_FILE_NAME, VALID_ARCHIVES
+from sabnzbd.constants import SCAN_FILE_NAME, VALID_ARCHIVES, VALID_NZB_FILES
 import sabnzbd.utils.rarfile as rarfile
 from sabnzbd.encoding import platform_encode
+from sabnzbd.decorators import NzbQueueLocker
 from sabnzbd.newsunpack import is_sevenfile, SevenZip
 import sabnzbd.nzbstuff as nzbstuff
 import sabnzbd.misc as misc
@@ -73,25 +74,30 @@ def is_archive(path):
             zf = zipfile.ZipFile(path)
             return 0, zf, '.zip'
         except:
+            logging.info(T('Cannot read %s'), path, exc_info=True)
             return -1, None, ''
-    elif rarfile.is_rarfile(path):
+    elif misc.is_rarfile(path):
         try:
-            zf = rarfile.RarFile(path)
             # Set path to tool to open it
             rarfile.UNRAR_TOOL = sabnzbd.newsunpack.RAR_COMMAND
+            zf = rarfile.RarFile(path)
             return 0, zf, '.rar'
         except:
+            logging.info(T('Cannot read %s'), path, exc_info=True)
             return -1, None, ''
     elif is_sevenfile(path):
         try:
             zf = SevenZip(path)
             return 0, zf, '.7z'
         except:
+            logging.info(T('Cannot read %s'), path, exc_info=True)
             return -1, None, ''
     else:
+        logging.info('Archive %s is not a real archive!', os.path.basename(path))
         return 1, None, ''
 
 
+@NzbQueueLocker
 def ProcessArchiveFile(filename, path, pp=None, script=None, cat=None, catdir=None, keep=False,
                        priority=None, url='', nzbname=None, password=None, nzo_id=None):
     """ Analyse ZIP file and create job(s).
@@ -99,7 +105,6 @@ def ProcessArchiveFile(filename, path, pp=None, script=None, cat=None, catdir=No
         returns (status, nzo_ids)
             status: -1==Error/Retry, 0==OK, 1==Ignore
     """
-    from sabnzbd.nzbqueue import add_nzo
     nzo_ids = []
     if catdir is None:
         catdir = cat
@@ -128,29 +133,36 @@ def ProcessArchiveFile(filename, path, pp=None, script=None, cat=None, catdir=No
                 try:
                     data = zf.read(name)
                 except:
+                    logging.error(T('Cannot read %s'), name, exc_info=True)
                     zf.close()
                     return -1, []
                 name = os.path.basename(name)
                 if data:
+                    nzo = None
                     try:
                         nzo = nzbstuff.NzbObject(name, pp, script, data, cat=cat, url=url,
                                                  priority=priority, nzbname=nzbname)
                         if not nzo.password:
                             nzo.password = password
+                    except (TypeError, ValueError):
+                        # Duplicate or empty, ignore
+                        pass
                     except:
-                        nzo = None
+                        # Something else is wrong, show error
+                        logging.error(T('Error while adding %s, removing'), name, exc_info=True)
+
                     if nzo:
                         if nzo_id:
                             # Re-use existing nzo_id, when a "future" job gets it payload
                             sabnzbd.nzbqueue.NzbQueue.do.remove(nzo_id, add_to_history=False)
                             nzo.nzo_id = nzo_id
                             nzo_id = None
-                        nzo_ids.append(add_nzo(nzo))
+                        nzo_ids.append(sabnzbd.nzbqueue.NzbQueue.do.add(nzo))
                         nzo.update_rating()
         zf.close()
         try:
             if not keep:
-                os.remove(path)
+                misc.remove_file(path)
         except:
             logging.error(T('Error removing %s'), misc.clip_path(path))
             logging.info("Traceback: ", exc_info=True)
@@ -162,6 +174,7 @@ def ProcessArchiveFile(filename, path, pp=None, script=None, cat=None, catdir=No
     return status, nzo_ids
 
 
+@NzbQueueLocker
 def ProcessSingleFile(filename, path, pp=None, script=None, cat=None, catdir=None, keep=False,
                       priority=None, nzbname=None, reuse=False, nzo_info=None, dup_check=True, url='',
                       password=None, nzo_id=None):
@@ -170,7 +183,6 @@ def ProcessSingleFile(filename, path, pp=None, script=None, cat=None, catdir=Non
         returns (status, nzo_ids)
             status: -2==Error/retry, -1==Error, 0==OK, 1==OK-but-ignorecannot-delete
     """
-    from sabnzbd.nzbqueue import add_nzo
     nzo_ids = []
     if catdir is None:
         catdir = cat
@@ -220,10 +232,12 @@ def ProcessSingleFile(filename, path, pp=None, script=None, cat=None, catdir=Non
         # Empty, but correct file
         return -1, nzo_ids
     except:
-        if data.find("<nzb") >= 0 and data.find("</nzb") < 0:
+        if data.find("<nzb") >= 0 > data.find("</nzb"):
             # Looks like an incomplete file, retry
             return -2, nzo_ids
         else:
+            # Something else is wrong, show error
+            logging.error(T('Error while adding %s, removing'), name, exc_info=True)
             return -1, nzo_ids
 
     if nzo:
@@ -231,11 +245,11 @@ def ProcessSingleFile(filename, path, pp=None, script=None, cat=None, catdir=Non
             # Re-use existing nzo_id, when a "future" job gets it payload
             sabnzbd.nzbqueue.NzbQueue.do.remove(nzo_id, add_to_history=False)
             nzo.nzo_id = nzo_id
-        nzo_ids.append(add_nzo(nzo, quiet=reuse))
+        nzo_ids.append(sabnzbd.nzbqueue.NzbQueue.do.add(nzo, quiet=reuse))
         nzo.update_rating()
     try:
         if not keep:
-            os.remove(path)
+            misc.remove_file(path)
     except:
         logging.error(T('Error removing %s'), misc.clip_path(path))
         logging.info("Traceback: ", exc_info=True)
@@ -344,7 +358,7 @@ class DirScanner(threading.Thread):
                     continue
 
                 ext = os.path.splitext(path)[1].lower()
-                candidate = ext in ('.nzb', '.gz', '.bz2') or ext in VALID_ARCHIVES
+                candidate = ext in VALID_NZB_FILES + VALID_ARCHIVES
                 if candidate:
                     try:
                         stat_tuple = os.stat(path)
