@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2019 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -29,6 +29,7 @@ from sabnzbd.constants import *
 import sabnzbd
 from sabnzbd.misc import to_units, split_host, time_format
 from sabnzbd.encoding import EmailFilter
+from sabnzbd.notifier import check_cat
 import sabnzbd.cfg as cfg
 
 
@@ -148,6 +149,8 @@ def send(message, email_to, test=None):
         else:
             logging.info("Notification e-mail successfully sent")
             return T('Email succeeded')
+    else:
+        return T('Cannot send, missing required data')
 
 
 def get_email_date():
@@ -216,6 +219,9 @@ def send_with_template(prefix, parm, test=None):
 
 def endjob(filename, cat, status, path, bytes, fail_msg, stages, script, script_output, script_ret, test=None):
     """ Send end-of-job email """
+    # Is it allowed?
+    if not check_cat('misc', cat, keyword='email') and not test:
+        return None
 
     # Translate the stage names
     tr = sabnzbd.api.Ttemplate
@@ -354,7 +360,7 @@ def _prepare_message(txt):
     msg.set_payload('\n'.join(payload), code)
 
     # Check for proper encoding, else call it explicitly
-    if not msg.has_key('Content-Transfer-Encoding'):
+    if 'Content-Transfer-Encoding' not in msg:
         encode_quopri(msg)
 
     return msg.as_string()
