@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2019 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -20,6 +20,7 @@ sabnzbd.encoding - Unicoded filename support
 """
 
 import locale
+import string
 from xml.sax.saxutils import escape
 from Cheetah.Filters import Filter
 
@@ -51,14 +52,6 @@ def change_fsys(value):
             auto_fsys()
 
 
-def reliable_unpack_names():
-    """ See if it is safe to rely on unrar names """
-    if sabnzbd.WIN32 or sabnzbd.DARWIN:
-        return True
-    else:
-        return gUTF
-
-
 def platform_encode(p):
     """ Return Unicode name, if not already Unicode, decode with UTF-8 or latin1 """
     if isinstance(p, str):
@@ -70,22 +63,6 @@ def platform_encode(p):
         return p
 
 
-def name_fixer(p):
-    """ Return Unicode name of 8bit ASCII string, first try UTF-8, then codepage, then cp1252 """
-    if isinstance(p, unicode):
-        return p
-    elif isinstance(p, str):
-        try:
-            return p.decode('utf-8')
-        except:
-            try:
-                return p.decode(codepage)
-            except:
-                return p.decode('cp1252', 'replace').replace('?', '!')
-    else:
-        return p
-
-
 def yenc_name_fixer(p):
     """ Return Unicode name of 8bit ASCII string, first try utf-8, then cp1252 """
     try:
@@ -94,19 +71,6 @@ def yenc_name_fixer(p):
         return p.decode('cp1252', errors='replace').replace('?', '!')
 
 
-def is_utf8(p):
-    """ Return True when p is UTF-8 or plain ASCII """
-    utf8 = True
-    try:
-        p.decode('ascii')
-    except:
-        try:
-            p.decode('utf-8')
-        except:
-            utf8 = False
-    return utf8
-
-
 def special_fixer(p):
     """ Return string appropriate for the platform.
         Also takes care of the situation where a non-Windows/UTF-8 system
@@ -145,16 +109,8 @@ def unicoder(p, force=False):
         return unicode(str(p))
 
 
-def unicode2local(p):
-    """ Convert Unicode filename to appropriate local encoding
-        Leave ? characters for uncovertible characters
-    """
-    return p
-
-
 def xml_name(p, keep_escape=False, encoding=None):
     """ Prepare name for use in HTML/XML contect """
-
     if isinstance(p, unicode):
         pass
     elif isinstance(p, str):
@@ -173,19 +129,6 @@ def xml_name(p, keep_escape=False, encoding=None):
         return escape(p).encode('ascii', 'xmlcharrefreplace')
 
 
-def encode_for_xml(ustr, encoding='ascii'):
-    """ Encode unicode_data for use as XML or HTML, with characters outside
-        of the encoding converted to XML numeric character references.
-    """
-    if isinstance(ustr, unicode):
-        pass
-    elif isinstance(ustr, str):
-        ustr = ustr.decode(codepage, 'replace')
-    else:
-        ustr = unicode(str(ustr))
-    return ustr.encode(encoding, 'xmlcharrefreplace')
-
-
 class LatinFilter(Filter):
     """ Make sure Cheetah gets only Unicode strings """
 
@@ -228,10 +171,10 @@ class EmailFilter(Filter):
 ################################################################################
 #
 # Map CodePage-850 characters to Python's pseudo-Unicode 8bit ASCII
-#
 # Use to transform 8-bit console output to plain Python strings
+# For example for unrar and par2 output
 #
-import string
+
 TAB_850 = \
     "\x80\x81\x82\x83\x84\x85\x86\x87\x88\x89\x8A\x8B\x8C\x8D\x8E\x8F" \
     "\x90\x91\x92\x93\x94\x95\x96\x97\x98\x99\x9A\x9B\x9C\x9D\x9E\x9F" \
@@ -244,7 +187,7 @@ TAB_850 = \
 
 TAB_LATIN = \
     "\xC7\xFC\xE9\xE2\xE4\xE0\xE5\xE7\xEA\xEB\xE8\xEF\xEE\xEC\xC4\xC5" \
-    "\xC9\xE6\xC6\xF4\xF6\xF2\xFB\xF9\xFF\xD6\xDC\xF8\xA3\xD8\xD7\x66" \
+    "\xC9\xE6\xC6\xF4\xF6\xF2\xFB\xF9\xFF\xD6\xDC\xF8\xA3\xD8\xD7\x83" \
     "\xE1\xED\xF3\xFA\xF1\xD1\xAA\xBA\xBF\xAE\xAC\xDB\xBC\xA1\xAB\xBB" \
     "\x7E\x7E\x7E\x7E\x7E\xC1\xC2\xC0\xA9\x7E\x7E\x7E\x7E\xA2\xA5\x7E" \
     "\x7E\x7E\x7E\x7E\x7E\x7E\xE3\xc3\x7E\x7E\x7E\x7E\x7E\x7E\x7E\xA4" \
@@ -252,32 +195,21 @@ TAB_LATIN = \
     "\xD3\xDF\xD4\xD2\xF5\xD5\xB5\xFE\xDE\xDA\xDB\xD9\xFD\xDD\xAF\xB4" \
     "\xAD\xB1\x5F\xBE\xB6\xA7\xF7\xB8\xB0\xA8\xB7\xB9\xB3\xB2\x7E\xA0"
 
-gTABLE_850_LATIN = string.maketrans(TAB_850, TAB_LATIN)
-gTABLE_LATIN_850 = string.maketrans(TAB_LATIN, TAB_850)
-
 
 def TRANS(p):
     """ For Windows: Translate CP850 to Python's Latin-1 and return in Unicode
         Others: return original string
     """
-    global gTABLE_850_LATIN
     if sabnzbd.WIN32:
-        return p.translate(gTABLE_850_LATIN).decode('cp1252', 'replace')
+        if p:
+            return p.translate(string.maketrans(TAB_850, TAB_LATIN)).decode('cp1252', 'replace')
+        else:
+            # translate() fails on empty or None strings
+            return ''
     else:
         return unicoder(p)
 
 
-def UNTRANS(p):
-    """ For Windows: Translate Python's Latin-1 to CP850
-        Others: return original string
-    """
-    global gTABLE_LATIN_850
-    if sabnzbd.WIN32:
-        return p.encode('cp1252', 'replace').translate(gTABLE_LATIN_850)
-    else:
-        return p
-
-
 def fixup_ff4(p):
     """ Fix incompatibility between CherryPy and Firefox-4 on OSX,
         where a filename contains &#xx; encodings
@@ -349,7 +281,7 @@ def deunicode(p):
             except:
                 return p
     else:
-        return p
+        return str(p)
 
 
 auto_fsys()
