@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2019 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -25,51 +25,34 @@ from threading import Thread
 from nntplib import NNTPPermanentError
 import time
 import logging
-import re
-import select
+import ssl
 
 import sabnzbd
 from sabnzbd.constants import *
 import sabnzbd.cfg
+from sabnzbd.misc import nntp_to_msg, probablyipv4, probablyipv6
 
-import threading
-_RLock = threading.RLock
-del threading
-
-# Import SSL if available
-if sabnzbd.HAVE_SSL:
-    import ssl
-    if sabnzbd.HAVE_SSL_CONTEXT:
-        WantReadError = ssl.SSLWantReadError
-        CertificateError = ssl.CertificateError
-    else:
-        WantReadError = ssl.SSLError
-        CertificateError = ssl.SSLError
+# Have to make errors available under Python <2.7.9
+if sabnzbd.HAVE_SSL_CONTEXT:
+    WantReadError = ssl.SSLWantReadError
+    CertificateError = ssl.CertificateError
 else:
-    # Dummy class so this exception is ignored by clients without ssl installed
-    class WantReadError(Exception):
-        def __init__(self, value):
-            self.parameter = value
-        def __str__(self):
-            return repr(self.parameter)
-    class CertificateError(Exception):
-        def __init__(self, value):
-            self.parameter = value
-        def __str__(self):
-            return repr(self.parameter)
-
+    WantReadError = ssl.SSLError
+    CertificateError = ssl.SSLError
 
+# Set pre-defined socket timeout
 socket.setdefaulttimeout(DEF_TIMEOUT)
 
 # getaddrinfo() can be very slow. In some situations this can lead
 # to delayed starts and timeouts on connections.
 # Because of this, the results will be cached in the server object.
 
+
 def _retrieve_info(server):
     """ Async attempt to run getaddrinfo() for specified server """
+    logging.debug('Retrieving server address information for %s', server.host)
     info = GetServerParms(server.host, server.port)
-
-    if info is None:
+    if not info:
         server.bad_cons += server.threads
     else:
         server.bad_cons = 0
@@ -116,7 +99,7 @@ def GetServerParms(host, port):
             except:
                 # Nothing found!
                 pass
-        return None
+        return False
 
 
 def get_ssl_version(sock):
@@ -129,11 +112,10 @@ def get_ssl_version(sock):
 
 
 def con(sock, host, port, sslenabled, write_fds, nntp):
-    if 0: assert isinstance(nntp, NNTP) # Assert only for debug purposes
     try:
         sock.connect((host, port))
         sock.setblocking(0)
-        if sslenabled and sabnzbd.HAVE_SSL:
+        if sslenabled:
             # Log SSL/TLS info
             logging.info("%s@%s: Connected using %s (%s)",
                                               nntp.nw.thrdnum, nntp.nw.server.host, get_ssl_version(sock), sock.cipher()[0])
@@ -164,26 +146,11 @@ def con(sock, host, port, sslenabled, write_fds, nntp):
             nntp.error(e)
 
 
-
-
-def probablyipv4(ip):
-    if ip.count('.') == 3 and re.sub('[0123456789.]', '', ip) == '':
-        return True
-    else:
-        return False
-
-
-def probablyipv6(ip):
-    if ip.count(':') >= 2 and re.sub('[0123456789abcdefABCDEF:]', '', ip) == '':
-        return True
-    else:
-        return False
-
-
 class NNTP(object):
+    # Pre-define attributes to save memory
+    __slots__ = ('host', 'port', 'nw', 'blocking', 'error_msg', 'sock')
 
-    def __init__(self, host, port, info, sslenabled, send_group, nw, user=None, password=None, block=False, write_fds=None):
-        if 0: assert isinstance(nw, NewsWrapper) # Assert only for debug purposes
+    def __init__(self, host, port, info, sslenabled, nw, block=False, write_fds=None):
         self.host = host
         self.port = port
         self.nw = nw
@@ -191,10 +158,7 @@ class NNTP(object):
         self.error_msg = None
 
         if not info:
-            if block:
-                info = GetServerParms(host, port)
-            else:
-                raise socket.error(errno.EADDRNOTAVAIL, "Address not available - Check for internet or DNS problems")
+            raise socket.error(errno.EADDRNOTAVAIL, "Address not available - Check for internet or DNS problems")
 
         af, socktype, proto, canonname, sa = info[0]
 
@@ -204,34 +168,34 @@ class NNTP(object):
         if probablyipv6(host):
             af = socket.AF_INET6
 
-        if sslenabled and sabnzbd.HAVE_SSL:
+        if sslenabled:
             # Use context or just wrapper
             if sabnzbd.HAVE_SSL_CONTEXT:
                 # Setup the SSL socket
                 ctx = ssl.create_default_context()
 
+                if sabnzbd.cfg.require_modern_tls():
+                    # We want a modern TLS (1.2 or higher), so we disallow older protocol versions (<= TLS 1.1)
+                    ctx.options |= ssl.OP_NO_SSLv2 | ssl.OP_NO_SSLv3 | ssl.OP_NO_TLSv1 | ssl.OP_NO_TLSv1_1
+
                 # Only verify hostname when we're strict
-                if(nw.server.ssl_verify < 2):
+                if nw.server.ssl_verify < 2:
                     ctx.check_hostname = False
                 # Certificates optional
-                if(nw.server.ssl_verify == 0):
+                if nw.server.ssl_verify == 0:
                     ctx.verify_mode = ssl.CERT_NONE
 
                 # Did the user set a custom cipher-string?
-                if(sabnzbd.cfg.ssl_ciphers()):
+                if nw.server.ssl_ciphers:
                     # At their own risk, socket will error out in case it was invalid
-                    ctx.set_ciphers(sabnzbd.cfg.ssl_ciphers())
+                    ctx.set_ciphers(nw.server.ssl_ciphers)
 
                 self.sock = ctx.wrap_socket(socket.socket(af, socktype, proto), server_hostname=str(nw.server.host))
             else:
                 # Ciphers have to be None, if set to empty-string it will fail on <2.7.9
-                ciphers = sabnzbd.cfg.ssl_ciphers() if sabnzbd.cfg.ssl_ciphers() else None
+                ciphers = nw.server.ssl_ciphers if nw.server.ssl_ciphers else None
                 # Use a regular wrapper, no certificate validation
                 self.sock = ssl.wrap_socket(socket.socket(af, socktype, proto), ciphers=ciphers)
-
-        elif sslenabled and not sabnzbd.HAVE_SSL:
-            logging.error(T('Error importing OpenSSL module. Connecting with NON-SSL'))
-            self.sock = socket.socket(af, socktype, proto)
         else:
             self.sock = socket.socket(af, socktype, proto)
 
@@ -244,7 +208,7 @@ class NNTP(object):
                 # if blocking (server test) only wait for 15 seconds during connect until timeout
                 self.sock.settimeout(15)
                 self.sock.connect((self.host, self.port))
-                if sslenabled and sabnzbd.HAVE_SSL:
+                if sslenabled:
                     # Log SSL/TLS info
                     logging.info("%s@%s: Connected using %s (%s)",
                                               self.nw.thrdnum, self.nw.server.host, get_ssl_version(self.sock), self.sock.cipher()[0])
@@ -268,18 +232,32 @@ class NNTP(object):
             finally:
                 self.error(e)
 
-
     def error(self, error):
-        if 'SSL23_GET_SERVER_HELLO' in str(error) or 'SSL3_GET_RECORD' in str(error):
+        raw_error_str = str(error)
+        if 'SSL23_GET_SERVER_HELLO' in str(error) or 'SSL3_GET_RECORD' in raw_error_str:
             error = T('This server does not allow SSL on this port')
 
         # Catch certificate errors
-        if type(error) == CertificateError or 'CERTIFICATE_VERIFY_FAILED' in str(error):
-            error = T('Server %s uses an untrusted certificate [%s]') % (self.nw.server.host, str(error))
-            error += ' - https://sabnzbd.org/certificate-errors'
+        if type(error) == CertificateError or 'CERTIFICATE_VERIFY_FAILED' in raw_error_str:
+            # Log the raw message for debug purposes
+            logging.info('Certificate error for host %s: %s', self.nw.server.host, raw_error_str)
+
+            # Try to see if we should catch this message and provide better text
+            if 'hostname' in raw_error_str:
+                raw_error_str = T('Certificate hostname mismatch: the server hostname is not listed in the certificate. This is a server issue.')
+            elif 'certificate verify failed' in raw_error_str:
+                raw_error_str = T('Certificate not valid. This is most probably a server issue.')
+
+            # Reformat error
+            error = T('Server %s uses an untrusted certificate [%s]') % (self.nw.server.host, raw_error_str)
+            error = '%s - %s: %s' % (error, T('Wiki'), 'https://sabnzbd.org/certificate-errors')
+
             # Prevent throwing a lot of errors or when testing server
-            if error not in self.nw.server.warning and self.nw.server.id != -1:
+            if error not in self.nw.server.warning and not self.blocking:
                 logging.error(error)
+            # Pass to server-test
+            if self.blocking:
+                raise CertificateError(error)
 
         # Blocking = server-test, pass directly to display code
         if self.blocking:
@@ -293,6 +271,9 @@ class NNTP(object):
 
 
 class NewsWrapper(object):
+    # Pre-define attributes to save memory
+    __slots__ = ('server', 'thrdnum', 'blocking', 'timeout', 'article', 'data', 'lines', 'last_line',  'nntp',
+                 'recv', 'connected', 'user_sent', 'pass_sent', 'group', 'user_ok', 'pass_ok', 'force_login')
 
     def __init__(self, server, thrdnum, block=False):
         self.server = server
@@ -301,8 +282,9 @@ class NewsWrapper(object):
 
         self.timeout = None
         self.article = None
-        self.data = ''
+        self.data = []
         self.lines = []
+        self.last_line = ''
 
         self.nntp = None
         self.recv = None
@@ -318,12 +300,24 @@ class NewsWrapper(object):
         self.pass_ok = False
         self.force_login = False
 
-    def init_connect(self, write_fds):
-        self.nntp = NNTP(self.server.hostip, self.server.port, self.server.info, self.server.ssl,
-                         self.server.send_group, self, self.server.username, self.server.password,
-                         self.blocking, write_fds)
-        self.recv = self.nntp.sock.recv
+    @property
+    def status_code(self):
+        """ Shorthand to get the code """
+        try:
+            return self.data[0][:3]
+        except:
+            return ''
 
+    def init_connect(self, write_fds):
+        # Server-info is normally requested by initialization of
+        # servers in Downloader, but not when testing servers
+        if self.blocking and not self.server.info:
+            self.server.info = GetServerParms(self.server.host, self.server.port)
+
+        # Construct NNTP object and shorthands
+        self.nntp = NNTP(self.server.hostip, self.server.port, self.server.info, self.server.ssl,
+                         self, self.blocking, write_fds)
+        self.recv = self.nntp.sock.recv
         self.timeout = time.time() + self.server.timeout
 
     def finish_connect(self, code):
@@ -337,7 +331,7 @@ class NewsWrapper(object):
         if code in ('501',) and self.user_sent:
             # Change to a sensible text
             code = '481'
-            self.lines[0] = T('Authentication failed, check username/password.')
+            self.data[0] = T('Authentication failed, check username/password.')
             self.user_ok = True
             self.pass_sent = True
 
@@ -350,10 +344,11 @@ class NewsWrapper(object):
             self.pass_ok = False
 
         if code in ('400', '502'):
-            raise NNTPPermanentError(self.lines[0])
+            raise NNTPPermanentError(nntp_to_msg(self.data))
         elif not self.user_sent:
             command = 'authinfo user %s\r\n' % self.server.username
             self.nntp.sock.sendall(command)
+            self.data = []
             self.user_sent = True
         elif not self.user_ok:
             if code == '381':
@@ -368,11 +363,12 @@ class NewsWrapper(object):
         if self.user_ok and not self.pass_sent:
             command = 'authinfo pass %s\r\n' % self.server.password
             self.nntp.sock.sendall(command)
+            self.data = []
             self.pass_sent = True
         elif self.user_ok and not self.pass_ok:
             if code != '281':
                 # Assume that login failed (code 481 or other)
-                raise NNTPPermanentError(self.lines[0])
+                raise NNTPPermanentError(nntp_to_msg(self.data))
             else:
                 self.connected = True
 
@@ -382,19 +378,21 @@ class NewsWrapper(object):
         self.timeout = time.time() + self.server.timeout
         if precheck:
             if self.server.have_stat:
-                command = 'STAT <%s>\r\n' % (self.article.article)
+                command = 'STAT <%s>\r\n' % self.article.article
             else:
-                command = 'HEAD <%s>\r\n' % (self.article.article)
+                command = 'HEAD <%s>\r\n' % self.article.article
         elif self.server.have_body:
-            command = 'BODY <%s>\r\n' % (self.article.article)
+            command = 'BODY <%s>\r\n' % self.article.article
         else:
-            command = 'ARTICLE <%s>\r\n' % (self.article.article)
+            command = 'ARTICLE <%s>\r\n' % self.article.article
         self.nntp.sock.sendall(command)
+        self.data = []
 
     def send_group(self, group):
         self.timeout = time.time() + self.server.timeout
-        command = 'GROUP %s\r\n' % (group)
+        command = 'GROUP %s\r\n' % group
         self.nntp.sock.sendall(command)
+        self.data = []
 
     def recv_chunk(self, block=False):
         """ Receive data, return #bytes, done, skip """
@@ -420,35 +418,62 @@ class NewsWrapper(object):
                     # time.sleep(0.0001)
                     continue
                 else:
-                    return (0, False, True)
+                    return 0, False, True
 
-        self.data += chunk
-        new_lines = self.data.split('\r\n')
-        # See if incorrect newline-only was used
-        # Do this as a special case to prevent using extra memory
-        # for normal articles
-        if len(new_lines) == 1 and '\r' not in self.data:
-            new_lines = self.data.split('\n')
+        # Data is processed differently depending on C-yEnc version
+        if sabnzbd.decoder.SABYENC_ENABLED:
+            # Append so we can do 1 join(), much faster than multiple!
+            self.data.append(chunk)
 
-        self.data = new_lines.pop()
+            # Official end-of-article is ".\r\n" but sometimes it can get lost between 2 chunks
+            chunk_len = len(chunk)
+            if chunk[-5:] == '\r\n.\r\n':
+                return chunk_len, True, False
+            elif chunk_len < 5 and len(self.data) > 1:
+                # We need to make sure the end is not split over 2 chunks
+                # This is faster than join()
+                combine_chunk = self.data[-2][-5:] + chunk
+                if combine_chunk[-5:] == '\r\n.\r\n':
+                    return chunk_len, True, False
 
-        # Already remove the starting dots
-        for i in xrange(len(new_lines)):
-            if new_lines[i][:2] == '..':
-                new_lines[i] = new_lines[i][1:]
-        self.lines.extend(new_lines)
-
-        if self.lines and self.lines[-1] == '.':
-            self.lines = self.lines[1:-1]
-            return (len(chunk), True, False)
+            # Still in middle of data, so continue!
+            return chunk_len, False, False
         else:
-            return (len(chunk), False, False)
+            self.last_line += chunk
+            new_lines = self.last_line.split('\r\n')
+            # See if incorrect newline-only was used
+            # Do this as a special case to prevent using extra memory
+            # for normal articles
+            if len(new_lines) == 1 and '\r' not in self.last_line:
+                new_lines = self.last_line.split('\n')
+
+            self.last_line = new_lines.pop()
+
+            # Already remove the starting dots
+            for i in xrange(len(new_lines)):
+                if new_lines[i][:2] == '..':
+                    new_lines[i] = new_lines[i][1:]
+            self.lines.extend(new_lines)
+
+            # For status-code purposes
+            if not self.data:
+                self.data.append(chunk)
+
+            if self.lines and self.lines[-1] == '.':
+                self.lines = self.lines[1:-1]
+                return len(chunk), True, False
+            else:
+                return len(chunk), False, False
 
     def soft_reset(self):
         self.timeout = None
         self.article = None
-        self.data = ''
+        self.clear_data()
+
+    def clear_data(self):
+        self.data = []
         self.lines = []
+        self.last_line = ''
 
     def hard_reset(self, wait=True, quit=True):
         if self.nntp:
