@@ -722,11 +722,13 @@ class NzbQueue(object):
         """ Get next article for jobs in the queue
             Not locked for performance, since it only reads the queue
         """
+        # Pre-calculate propagation delay
+        propagtion_delay = float(cfg.propagation_delay() * 60)
         for nzo in self.__nzo_list:
             # Not when queue paused and not a forced item
             if nzo.status not in (Status.PAUSED, Status.GRABBING) or nzo.priority == TOP_PRIORITY:
                 # Check if past propagation delay, or forced
-                if not cfg.propagation_delay() or nzo.priority == TOP_PRIORITY or (nzo.avg_stamp + float(cfg.propagation_delay() * 60)) < time.time():
+                if not propagtion_delay or nzo.priority == TOP_PRIORITY or (nzo.avg_stamp + propagtion_delay) < time.time():
                     if not nzo.server_in_try_list(server):
                         article = nzo.get_article(server, servers)
                         if article:
