@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2019 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -59,12 +59,6 @@ def stop():
             pass
 
 
-def del_feed(feed):
-    global __RSS
-    if __RSS:
-        __RSS.delete(feed)
-
-
 def run_feed(feed, download, ignoreFirst=False, force=False, readout=True):
     global __RSS
     if __RSS:
@@ -173,42 +167,22 @@ LOCK = threading.RLock()
 class RSSQueue(object):
 
     def __init__(self):
-        def check_str(p):
-            return p is None or p == '' or isinstance(p, basestring)
-
-        def check_int(p):
-            try:
-                int(p)
-                return True
-            except:
-                return False
-
         self.jobs = {}
         self.next_run = time.time()
         self.shutdown = False
 
         try:
-            defined = config.get_rss().keys()
-            feeds = sabnzbd.load_admin(RSS_FILE_NAME)
-            if type(feeds) == type({}):
-                for feed in feeds:
-                    if feed not in defined:
-                        logging.debug('Dropping obsolete data for feed "%s"', feed)
-                        continue
-                    self.jobs[feed] = {}
-                    for link in feeds[feed]:
-                        data = feeds[feed][link]
-                        # Consistency check on data
-                        try:
-                            item = feeds[feed][link]
-                            if not isinstance(item, dict) or not isinstance(item.get('title'), unicode):
-                                raise IndexError
-                            self.jobs[feed][link] = item
-                        except (KeyError, IndexError):
-                            logging.info('Incorrect entry in %s detected, discarding %s', RSS_FILE_NAME, item)
+            self.jobs = sabnzbd.load_admin(RSS_FILE_NAME)
+            if self.jobs:
+                for feed in self.jobs:
                     remove_obsolete(self.jobs[feed], self.jobs[feed].keys())
-        except IOError:
-            logging.debug('Cannot read file %s', RSS_FILE_NAME)
+        except:
+            logging.warning(T('Cannot read %s'), RSS_FILE_NAME)
+            logging.info("Traceback: ", exc_info=True)
+
+        # Storage needs to be dict
+        if not self.jobs:
+            self.jobs = {}
 
         # jobs is a NAME-indexed dictionary
         #    Each element is link-indexed dictionary
@@ -224,7 +198,6 @@ class RSSQueue(object):
         #           script : script
         #           prio : priority
         #           time : timestamp (used for time-based clean-up)
-        #           order : order in the RSS feed
         #           size : size in bytes
         #           age : age in datetime format as specified by feed
         #           season : season number (if applicable)
@@ -314,10 +287,10 @@ class RSSQueue(object):
 
                 status = feed_parsed.get('status', 999)
                 if status in (401, 402, 403):
-                    msg = T('Do not have valid authentication for feed %s') % feed
+                    msg = T('Do not have valid authentication for feed %s') % uri
                     logging.info(msg)
 
-                if status >= 500 and status <= 599:
+                if 500 <= status <= 599:
                     msg = T('Server side error (server code %s); could not get %s on %s') % (status, feed, uri)
                     logging.info(msg)
 
@@ -328,28 +301,31 @@ class RSSQueue(object):
                         msg = T('Server %s uses an untrusted HTTPS certificate') % get_urlbase(uri)
                         msg += ' - https://sabnzbd.org/certificate-errors'
                         logging.error(msg)
+                    elif 'href' in feed_parsed and feed_parsed['href'] != uri and 'login' in feed_parsed['href']:
+                        # Redirect to login page!
+                        msg = T('Do not have valid authentication for feed %s') % uri
                     else:
                         msg = T('Failed to retrieve RSS from %s: %s') % (uri, xml_name(msg))
                     logging.info(msg)
 
-                if not entries:
+                if not entries and not msg:
                     msg = T('RSS Feed %s was empty') % uri
                     logging.info(msg)
                 all_entries.extend(entries)
             entries = all_entries
 
+        # In case of a new feed
         if feed not in self.jobs:
             self.jobs[feed] = {}
         jobs = self.jobs[feed]
+
+        # Error in readout or now new readout
         if readout:
             if not entries:
                 return unicoder(msg)
         else:
             entries = jobs.keys()
-            # Sort in the order the jobs came from the feed
-            entries.sort(lambda x, y: jobs[x].get('order', 0) - jobs[y].get('order', 0))
 
-        order = 0
         # Filter out valid new links
         for entry in entries:
             if self.shutdown:
@@ -357,12 +333,8 @@ class RSSQueue(object):
 
             if readout:
                 try:
-                    link, category, size, age, season, episode = _get_link(uri, entry)
+                    link, infourl, category, size, age, season, episode = _get_link(entry)
                 except (AttributeError, IndexError):
-                    link = None
-                    category = u''
-                    size = 0L
-                    age = None
                     logging.info(T('Incompatible feed') + ' ' + uri)
                     logging.info("Traceback: ", exc_info=True)
                     return T('Incompatible feed')
@@ -381,6 +353,7 @@ class RSSQueue(object):
                         continue
             else:
                 link = entry
+                infourl = jobs[link].get('infourl', '')
                 category = jobs[link].get('orgcat', '')
                 if category in ('', '*'):
                     category = None
@@ -411,11 +384,9 @@ class RSSQueue(object):
                     n = 0
                     if ('F' in reTypes or 'S' in reTypes) and (not season or not episode):
                         season, episode = sabnzbd.newsunpack.analyse_show(title)[1:3]
-                        season = int_conv(season)
-                        episode = int_conv(episode)
 
                     # Match against all filters until an positive or negative match
-                    logging.debug('Size %s for %s', size, title)
+                    logging.debug('Size %s', size)
                     for n in xrange(regcount):
                         if reEnabled[n]:
                             if category and reTypes[n] == 'C':
@@ -511,14 +482,13 @@ class RSSQueue(object):
                     else:
                         star = first
                     if result:
-                        _HandleLink(jobs, link, title, size, age, season, episode, 'G', category, myCat, myPP, myScript,
-                                    act, star, order, priority=myPrio, rule=str(n))
+                        _HandleLink(jobs, feed, link, infourl, title, size, age, season, episode, 'G', category, myCat,
+                                    myPP, myScript, act, star, priority=myPrio, rule=str(n))
                         if act:
                             new_downloads.append(title)
                     else:
-                        _HandleLink(jobs, link, title, size, age, season, episode, 'B', category, myCat, myPP, myScript,
-                                    False, star, order, priority=myPrio, rule=str(n))
-            order += 1
+                        _HandleLink(jobs, feed, link, infourl, title, size, age, season, episode, 'B', category, myCat,
+                                    myPP, myScript, False, star, priority=myPrio, rule=str(n))
 
         # Send email if wanted and not "forced"
         if new_downloads and cfg.email_rss() and not force:
@@ -618,8 +588,8 @@ class RSSQueue(object):
         return ''
 
 
-def _HandleLink(jobs, link, title, size, age, season, episode, flag, orgcat, cat, pp, script, download, star,
-                order, priority=NORMAL_PRIORITY, rule=0):
+def _HandleLink(jobs, feed, link, infourl, title, size, age, season, episode, flag, orgcat, cat, pp, script,
+                download, star, priority=NORMAL_PRIORITY, rule=0):
     """ Process one link """
     if script == '':
         script = None
@@ -629,11 +599,11 @@ def _HandleLink(jobs, link, title, size, age, season, episode, flag, orgcat, cat
     jobs[link] = {}
     jobs[link]['title'] = title
     jobs[link]['url'] = link
+    jobs[link]['infourl'] = infourl
     jobs[link]['cat'] = cat
     jobs[link]['pp'] = pp
     jobs[link]['script'] = script
     jobs[link]['prio'] = str(priority)
-    jobs[link]['order'] = order
     jobs[link]['orgcat'] = orgcat
     jobs[link]['size'] = size
     jobs[link]['age'] = age
@@ -658,14 +628,11 @@ def _HandleLink(jobs, link, title, size, age, season, episode, flag, orgcat, cat
         else:
             jobs[link]['status'] = flag
 
-def _get_link(uri, entry):
+def _get_link(entry):
     """ Retrieve the post link from this entry
         Returns (link, category, size)
     """
-    link = None
-    category = ''
     size = 0L
-    uri = uri.lower()
     age = datetime.datetime.now()
 
     # Try standard link and enclosures first
@@ -679,6 +646,11 @@ def _get_link(uri, entry):
         except:
             pass
 
+    # GUID usually has URL to result on page
+    infourl = None
+    if entry.get('id') and entry.id != link and entry.id.startswith('http'):
+        infourl = entry.id
+
     if size == 0L:
         _RE_SIZE1 = re.compile(r'Size:\s*(\d+\.\d+\s*[KMG]{0,1})B\W*', re.I)
         _RE_SIZE2 = re.compile(r'\W*(\d+\.\d+\s*[KMG]{0,1})B\W*', re.I)
@@ -728,10 +700,10 @@ def _get_link(uri, entry):
                     except:
                         category = ''
 
-        return link, category, size, age, season, episode
+        return link, infourl, category, size, age, season, episode
     else:
         logging.warning(T('Empty RSS entry found (%s)'), link)
-        return None, '', 0L, None, 0, 0
+        return None, None, '', 0L, None, 0, 0
 
 
 def special_rss_site(url):
@@ -746,8 +718,11 @@ def ep_match(season, episode, expr, title=None):
     """
     m = _RE_SP.search(expr)
     if m:
+        # Make sure they are all integers for comparison
         req_season = int(m.group(1))
         req_episode = int(m.group(2))
+        season = int_conv(season)
+        episode = int_conv(episode)
         if season > req_season or (season == req_season and episode >= req_episode):
             if title:
                 show = expr[:m.start()].replace('.', ' ').replace('_', ' ').strip()
