@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2019 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -21,7 +21,6 @@ sabnzbd.sabtraylinux - System tray icon for Linux, inspired from the Windows one
 
 import gtk
 import gobject
-import cherrypy
 from time import sleep
 import subprocess
 from threading import Thread
@@ -78,8 +77,8 @@ class StatusIcon(Thread):
     # run this every updatefreq ms
     def run(self):
         self.sabpaused, bytes_left, bpsnow, time_left = api.fast_queue()
-        mb_left = to_units(bytes_left, dec_limit=1)
-        speed = to_units(bpsnow, dec_limit=1)
+        mb_left = to_units(bytes_left)
+        speed = to_units(bpsnow)
 
         if self.sabpaused:
             self.tooltip = T('Paused')
@@ -137,12 +136,13 @@ class StatusIcon(Thread):
         dialog.set_select_multiple(True)
 
         filter = gtk.FileFilter()
-        filter.set_name("*.nbz,*.nbz.gz,*.bz2,*.zip,*.rar")
-        filter.add_pattern("*.nzb*")
-        filter.add_pattern("*.nzb.gz")
-        filter.add_pattern("*.nzb.bz2")
+        filter.set_name("*.nzb,*.gz,*.bz2,*.zip,*.rar,*.7z")
+        filter.add_pattern("*.nzb")
+        filter.add_pattern("*.gz")
+        filter.add_pattern("*.bz2")
         filter.add_pattern("*.zip")
         filter.add_pattern("*.rar")
+        filter.add_pattern("*.7z")
         dialog.add_filter(filter)
 
         response = dialog.run()
@@ -169,9 +169,7 @@ class StatusIcon(Thread):
 
     def shutdown(self, icon):
         self.hover_text = T('Shutdown')
-        sabnzbd.halt()
-        cherrypy.engine.exit()
-        sabnzbd.SABSTOP = True
+        sabnzbd.shutdown_program()
 
     def pause(self):
         scheduler.plan_resume(0)
