@@ -542,10 +542,10 @@ SKIN_TEXT = {
     'opt-rating_filter_downvoted' : TT('More thumbs down than up'),
     'opt-rating_filter_keywords' : TT('Title keywords'),
     'explain-rating_filter_keywords' : TT('Comma separated list'),
-    'opt-load_balancing' : TT('Server load-balancing'),
-    'no-load-balancing' : TT('Prevent load-balancing'),
-    'load-balancing' : TT('Allow load-balancing'),
-    'load-balancing-happy-eyeballs' : TT('Allow load-balancing with optimization for IPv6'),
+    'opt-load_balancing' : TT('Server IP address selection'),
+    'no-load-balancing' : TT('First IP address'),
+    'load-balancing' : TT('Randomly selected IP address'),
+    'load-balancing-happy-eyeballs' : TT('Quickest IP address, preferring IPv6'),
     'explain-load_balancing' : TT('Useful if a newsserver has more than one IPv4/IPv6 address'),
 
 # Config->Server
