@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2019 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -21,7 +21,6 @@ sabnzbd.zconfig - bonjour/zeroconfig support
 
 import os
 import logging
-import cherrypy
 
 _HOST_PORT = (None, None)
 
@@ -80,12 +79,6 @@ def set_bonjour(host=None, port=None):
         return
 
     name = hostname()
-    if '.local' in name:
-        suffix = ''
-    else:
-        suffix = '.local'
-    if hasattr(cherrypy.wsgiserver, 'redirect_url'):
-        cherrypy.wsgiserver.redirect_url("https://%s%s:%s/sabnzbd" % (name, suffix, port))
     logging.debug('Try to publish in Bonjour as "%s" (%s:%s)', name, host, port)
     try:
         refObject = pybonjour.DNSServiceRegister(
@@ -95,11 +88,15 @@ def set_bonjour(host=None, port=None):
             domain=domain,
             host=zhost,
             port=int(port),
-            txtRecord=pybonjour.TXTRecord({'path': '/sabnzbd/'}),
+            txtRecord=pybonjour.TXTRecord({'path': cfg.url_base(),
+                                           'https': cfg.enable_https()}),
             callBack=_zeroconf_callback)
-    except sabnzbd.utils.pybonjour.BonjourError:
+    except sabnzbd.utils.pybonjour.BonjourError as e:
         _BONJOUR_OBJECT = None
-        logging.debug('Failed to start Bonjour service')
+        logging.debug('Failed to start Bonjour service: %s', str(e))
+    except:
+        _BONJOUR_OBJECT = None
+        logging.debug('Failed to start Bonjour service due to non-pybonjour related problem', exc_info=True)
     else:
         Thread(target=_bonjour_server, args=(refObject,))
         _BONJOUR_OBJECT = refObject
